# Generated by `rjournal_pdf_article()` using `knitr::purl()`: do not edit by hand
# Please edit rater.Rmd to modify this file

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
library(ggplot2)
library(knitr)
library(kableExtra)


## ----data-formats-balanced, echo = FALSE--------------------------------------
balanced_long <- data.frame(
  Item   = c(1, 1, 2, 2, 3, 3),
  Rater  = c(1, 2, 1, 2, 1, 2),
  Rating = c(3, 4, 2, 2, 2, 2)
) |> 
  kable() |> 
  add_header_above(c("(a) Long" = 3))

balanced_wide <- data.frame(
  Item      = c(1, 2, 3),
  "Rater 1" = c(3, 2, 2),
  "Rater 2" = c(4, 2, 2),
  check.names = FALSE
) |> 
  kable() |> 
  add_header_above(c("(b) Wide" = 3))

balanced_grouped <- data.frame(
  "Rater 1" = c(3, 2),
  "Rater 2" = c(4, 2),
  Tally     = c(1, 2),
  check.names = FALSE
) |> 
  kable() |> 
  add_header_above(c("(c) Wide (grouped)" = 3))

kables(list(balanced_long, balanced_wide, balanced_grouped), 
       caption = "Formats for balanced rating data.") |> 
  kable_styling(position = "center")


## ----data-formats-unbalanced, echo = FALSE------------------------------------
unbalanced_long <- data.frame(
  Item   = c(1, 1, 2, 2, 3, 3, 4, 5, 6),
  Rater  = c(1, 2, 1, 2, 1, 2, 1, 1, 2),
  Rating = c(3, 4, 2, 2, 2, 2, 3, 3, 4)
) |> 
  kable() |> 
  add_header_above(c("(a) Long" = 3))

unbalanced_wide <- data.frame(
  Item      = c(1, 2, 3,  4,  5,  6),
  "Rater 1" = c(3, 2, 2,  3,  3, NA),
  "Rater 2" = c(4, 2, 2, NA, NA,  4),
  check.names = FALSE
) |> 
  kable() |> 
  add_header_above(c("(b) Wide" = 3))

unbalanced_grouped <- data.frame(
  "Rater 1" = c(3, 2,  3, NA),
  "Rater 2" = c(4, 2, NA,  4),
  Tally     = c(1, 2,  2,  1),
  check.names = FALSE
) |> 
  kable() |> 
  add_header_above(c("(c) Wide (grouped)" = 3))

kables(list(unbalanced_long, unbalanced_wide, unbalanced_grouped), 
       caption = "Formats for unbalanced rating data. Missing values are indicated by ‘NA’.") |> 
  kable_styling(position = "center")


## ----relationships------------------------------------------------------------
#| echo: FALSE
#| fig.cap: "Relationships between models. Models coloured blue are
#| implemented in <a href='https://CRAN.R-project.org/package=rater'>rater</a>.
#| DS: Dawid–Skene. LCM: latent class model."
#| out.width: 100%
knitr::include_graphics("tikz/figrelationships.png")


## ----package-features, echo = FALSE-------------------------------------------
data.frame(
  "Package" = c("rater", "BayesLCA", "randomLCA", "poLCA"),
  "DS model" = c("Yes", "When $K = 2$", "When $K = 2$", "Yes"),
  "Fitting method" = c("Bayesian", "Bayesian", "Frequentist", "Frequentist"),
  "Response type" = c("Polytomous", "Binary", "Binary", "Polytomous"),
  "Repeated ratings" = c("Yes", "No", "No", "No"),
  "Data formats" = c("Wide, grouped, long", "Wide, grouped", "Wide, grouped", "Wide"),
  check.names = FALSE
) |> 
  kable(caption = "Features of
  \\CRANpkg{rater} and existing R
  packages for fitting latent class models. DS: Dawid--Skene.") |> 
  kable_styling(position = "center")


## ----load-rater---------------------------------------------------------------
library(rater)


## ----load-anesthesia-data-----------------------------------------------------
data("anesthesia", package = "rater")
head(anesthesia)


## ----model-fit-mcmc, message = FALSE, results = "hide", cache = TRUE----------
fit_1 <- rater(anesthesia, dawid_skene())


## ----model-fit-optim, message = FALSE, results = "hide", cache = TRUE---------
fit_2 <- rater(anesthesia, dawid_skene(), method = "optim")


## ----model-fit-optim-string, eval = FALSE-------------------------------------
#> fit_2 <- rater(anesthesia, "dawid_skene", method = "optim")


## ----fit-summary--------------------------------------------------------------
summary(fit_1)


## ----inpsecting-fitted-models, echo = FALSE-----------------------------------
data.frame(
  " " = c("`summary()`", "`point_estimate()`", "`posterior_interval()`", 
          "`posterior_samples()`", "`mcmc_diagnostics()`", 
          "`class_probabilities()`"), 
  "MCMC mode" = c("Basic inforamtion about the fitted model", 
                  "Posterior means for $\\pi$ and $\\theta$, posterior modes for $z$",
                  "Credible intervals for $\\pi$ and $\\theta$", 
                  "MCMC draws for $\\pi$ and $\\theta$",
                  "MCMC convergence diagnostics for $\\pi$ and $\\theta$",
                  "Posterior distribution for $z$"), 
  "Optimisation mode" = c("Basic inforamtion about the fitted model",
                          "Posterior modes for all quantities", 
                          "N/A", 
                          "N/A", 
                          "N/A",
                          "Posterior distribution for $z$ conditional on the
                          posterior modes for $\\pi$ and $\\theta$"), 
  check.names = FALSE) |> 
  kable(caption = "Methods to inspect fitted models and what values they return.") |> 
  kable_styling(position = "center")


## ----point-esimate-z----------------------------------------------------------
point_estimate(fit_1, "z")


## ----point-estimate-pi--------------------------------------------------------
point_estimate(fit_1, "pi")


## ----credint-theta------------------------------------------------------------
head(posterior_interval(fit_1, 0.8, "theta"))


## ----draws-pi-----------------------------------------------------------------
head(posterior_samples(fit_1, "pi")$pi)


## ----classprob----------------------------------------------------------------
head(class_probabilities(fit_1))


## ----plot-theta---------------------------------------------------------------
#| fig.cap: "Visual representation of the inferred parameters in the error
#|     matrices ($\\theta$) for the Dawid--Skene model fitted via MCMC to the
#|     anaesthesia dataset. The values shown are posterior means, with each
#|     cell shaded on a gradient from white (value close to 0) to dark blue
#|     (value close to 1)."
#| fig.height: 3.8
#| fig.width: 5.5
#| fig.align: "center"
plot(fit_1, "raters")


## ----plot-z-------------------------------------------------------------------
#| fig.cap: "Visualisation of the inferred probability of each latent class, for
#|     a selected subset of items, for the Dawid--Skene model fitted via MCMC to
#|     the anaesthesia dataset."
#| fig.height: 4
#| fig.width: 4
#| fig.align: "center"
plot(fit_1, "latent_class", item_index = c(2, 3, 12, 36, 38))


## ----plot-theta-fit2----------------------------------------------------------
#| echo: FALSE
#| fig.cap: "Visual representation of the inferred parameters in the error
#|     matrices ($\\theta$) for the Dawid--Skene model fitted via optimisation
#|     to the anaesthesia dataset.  Compare with \\autoref{fig:plot-theta},
#|     which used MCMC instead of optimisation."
#| fig.height: 3.8
#| fig.width: 5.5
#| fig.align: "center"
plot(fit_2, "raters")


## ----plot-z-fit2--------------------------------------------------------------
#| echo: FALSE
#| fig.cap: "Visualisation of the inferred probability of each latent class, for
#|     a selected subset of items, for the Dawid--Skene model fitted via
#|     optimisation to the anaesthesia dataset.  Compare with
#|     \\autoref{fig:plot-z}, which used MCMC instead of optimisation."
#| fig.height: 4
#| fig.width: 4
#| fig.align: "center"
plot(fit_2, "latent_class", item_index = c(2, 3, 12, 36, 38))


## ----plot-pi------------------------------------------------------------------
#| fig.cap: "Visualisation of the inferred population prevalence parameters,
#|     with 90\\% credible intervals, for the Dawid--Skene model fitted to the
#|     anaesthesia dataset."
#| fig.height: 3
#| fig.width: 5.5
#| fig.align: "center"
plot(fit_1, "prevalence")


## ----model-fit-different-prior, message = FALSE, results = "hide", cache = TRUE----
diff_alpha_fit <- rater(anesthesia, dawid_skene(alpha = rep(10, 4)))


## ----model-fit-different-model, message = FALSE, results = "hide", cache = TRUE----
diff_model_fit <- rater(anesthesia, class_conditional_dawid_skene())


## ----plot-theta-diff-model----------------------------------------------------
#| fig.cap: "Visual representation of the inferred parameters in the error
#|     matrices ($\\theta$) for the class-conditional Dawid--Skene model fitted
#|     via MCMC to the anaesthesia dataset.  Compare with
#|     \\autoref{fig:plot-theta}, which used the standard Dawid--Skene model."
#| fig.height: 3.8
#| fig.width: 5.5
#| fig.align: "center"
plot(diff_model_fit, "theta")


## ----load-caries-data---------------------------------------------------------
data("caries", package = "rater")
head(caries)


## ----model-fit-caries, message = FALSE, results = "hide", cache = TRUE--------
fit3 <- rater(caries, dawid_skene(), data_format = "grouped")


## ----plot-theta-caries--------------------------------------------------------
#| fig.cap: "Visual representation of the inferred parameters in the error
#|     matrices ($\\theta$) for the Dawid--Skene model fitted via MCMC to the
#|     caries dataset."
#| fig.height: 3.8
#| fig.width: 5.5
#| fig.align: "center"
plot(fit3, "raters")


## ----run-mcmc_diagnostics-----------------------------------------------------
head(mcmc_diagnostics(fit_1))


## ----plot-rstan-traceplot-----------------------------------------------------
#| fig.cap: "A trace plot for the $\\pi_1$ parameter from the Dawid--Skene
#|     model fitted via MCMC to the anaesthesia dataset."
#| fig.height: 3.2
#| fig.width: 5.0
#| fig.align: "center"
stan_fit <- get_stanfit(fit_1)
rstan::traceplot(stan_fit, pars = "pi[1]")


## ----calculate-posterior-predictive-check-statistics, cache = TRUE------------
class2prop <- function() {
  simdata <- posterior_predict(fit_1, anesthesia[, 1:2])
  sum(simdata$rating == 2) / nrow(simdata)
}
ppc_statistics <- replicate(1000, class2prop())
head(ppc_statistics)


## ----plot-ppcs----------------------------------------------------------------
#| fig.cap: "An example of a posterior predictive check for the Dawid--Skene
#|     model fitted via MCMC to the anaesthesia dataset.  Shown is a histogram
#|     of 1,000 simulated datasets from the posterior predictive distribution,
#|     each summarised by the proportion of ratings that are class 2.  The
#|     vertical black line shows the proportion of ratings that are class 2 in
#|     the original dataset, for comparison against the histogram."
#| fig.height: 3.8
#| fig.width: 5.5
#| fig.align: "center"
ggplot(data.frame(prop_class_two = ppc_statistics), aes(prop_class_two)) +
  geom_histogram(binwidth = 0.01, fill = "steelblue", colour = "black") +
  geom_vline(xintercept = sum(anesthesia$rating == 2) / nrow(anesthesia),
             colour = "black", linewidth = 2) +
  theme_bw() +
  coord_cartesian(xlim = c(0.1, 0.6)) +
  labs(
    x = "Proportion of class 2 ratings",
    y = "Count"
  )


## ----loo-ds-vs-ccds, message = FALSE, warning = FALSE, cache = TRUE-----------
loo(fit_1)
loo(diff_model_fit)
loo_compare(loo(fit_1), loo(diff_model_fit))

